package ExprModel;

// ExprModel -- here, we model an expression in the pi calculus, using
// a tree of java structures.
//
// We use the abstract base class "expr", and every type of thing (par, sum,
// xmit, ...) descends from this.
//
// The state is an expr.
//

public class State extends Model.State
{ Expr e;
  public String toString() {return e.toString();}
  public Model.State Play(Model.Operators ops) throws Exception {return e.Play(ops);}
}


abstract class Expr
{ public abstract Model.State Play(Model.Operators ops) throws Exception;
  public abstract String toString();
  public String toTightString() {return toString();}
}

class Exprs extends java.util.Vector
{ Expr exprAt(int i) {return (Expr)elementAt(i);}
  public String toString(String sep)
  { String s="";
    for (int i=0; i<size(); i++) {if (i!=0) s=s+sep; s=s+exprAt(i).toString();}
    return s;
  }
}


class Name
{ String n;
  public Name() { }
  public String toString() {return n;}
}

class Names extends java.util.Vector
{ public final synchronized Name nameAt(int i)
  { return (Name)elementAt(i);
  }
  public String toStringList()
  { String s="";
    for (int i=0; i<size(); i++) {if (i!=0) s=s+","; s=s+nameAt(i).toString();}
    return s;
  }
}






class NilExpr extends Expr
{ public NilExpr() { }
  public Model.State Play(Model.Operators ops) throws Exception {return ops.Nil();}
  public String toString() {return "0";}
}

class FusionExpr extends Expr
{ Name n1,n2;
  public Model.State Play(Model.Operators ops) throws Exception {return ops.Fus(n1.n,n2.n);}
  public String toString() {return n1.n+"="+n2.n;}
}

class ParExpr extends Expr
{ Exprs es;
  public Model.State Play(Model.Operators ops) throws Exception
  { if (es==null || es.size()==0) return ops.Nil();
    Model.State s=es.exprAt(0).Play(ops);
    for (int i=1; i<es.size(); i++) {Model.State s2=es.exprAt(i).Play(ops); s=ops.Par(s,s2);}
    return s;
  }
  public String toString() {return es.toString(" | ");}
  public String toTightString() {return "("+toString()+")";}
}


class SumExpr extends Expr
{ Exprs es;
  public Model.State Play(Model.Operators ops) throws Exception
  { if (es==null || es.size()==0) return ops.Nil();
    Model.State s=es.exprAt(0).Play(ops);
    for (int i=1; i<es.size(); i++) {Model.State s2=es.exprAt(i).Play(ops); s=ops.Sum(s,s2);}
    return s;
  }
  public String toString() {return es.toString(" + ");}
  public String toTightString() {return "("+toString()+")";}
}


class ResExpr extends Expr
{ Names n;
  Expr e;
  public ResExpr() { }
  public Model.State Play(Model.Operators ops) throws Exception
  { Model.State s=e.Play(ops);
    if (n==null || n.size()==0) return s;
    String[] rs=new String[n.size()]; for (int i=0; i<n.size(); i++) rs[i]=n.nameAt(i).n;
    s=ops.Res(rs,s);
    return s;
  }
  public String toString() {return "("+n.toStringList()+")"+e.toTightString();}
}


class RepExpr extends Expr
{ Expr e;
  public Model.State Play(Model.Operators ops) throws Exception
  { Model.State s=e.Play(ops);
    s=ops.Rep(s);
    return s;
  }
  public String toString() {return "!"+e.toTightString();}
}


class XmitExpr extends Expr
{ Name chan; boolean issend;
  String[] dat; boolean isbound;
  Expr e;
  public XmitExpr() {chan=null; e=null;}
  public Model.State Play(Model.Operators ops) throws Exception
  { Model.State s=e.Play(ops);
    if (issend) s=ops.Snd(chan.n,dat,isbound,s);
    else s=ops.Rcv(chan.n,dat,isbound,s);
    return s;
  }
  public String toString()
  { String s=chan.toString();
    if (e!=null) s=s+"."+e.toTightString();
    return s;
  }
}


